//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* coord.cpp: coordinates and numeric parameters                          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>

#include <ask_simple_value.h> // for ASK flags
#include <readln.h>

using namespace std;

#include "mystd.h"
#include "coord.h"

//**************************************************************************//
//  class CCoord                                                            //
//**************************************************************************//

CCoord::CCoord( const string in )
{
    uint i, j, np, n;
    n = in.size();
    if (in[n-1]!=')')
        throw "input must terminate with ')'";
    for(j=n-2, np=1; j!=(uint)-1; j--) {
        if (in[j]==')') 
            np++;
        else if (in[j]=='(') {
            np--;
            if (np==0) break;
        }
    }
    if (np!=0)
        throw "input must contain balanced parentheses";
    for (i=j; i>0 && strchr(" \t",in[i-1]); i--) ;
    if (i==0)
        throw "input must contain a non-empty coordinate name";
    name = in.substr(0,i);
    unit = in.substr(j+1, n-j-2);
}

void CCoord::ask_and_set( const string& quest )
{
    string prompt = quest;
    if( name!="" )
        prompt += " [" + str() + "]";
    prompt += " ? ";
    for( ;; ) {
        string resp = NRead::readln( prompt );
        if        ( resp == "\\q" ) {
            throw "user escape to main menu";
        } else if ( resp == "" ) {
            if ( name!="" )
                return; // keep current value
            cout << "no default available\n";
        } else if ( resp == "?" ) {
            cout << "required input type: coordinate name and unit\n";
            cout << "input must be in the form: name (unit)\n";
            if( name!="" )
                cout << "just press ENTER to let current value unchanged ["
                     << str() << "]\n";
        } else {
            uint i, j, np, n;
            n = resp.size();
            if (resp[n-1]!=')') {
                cout << "input must terminate with ')'\n";
                continue;
            }
            for(j=n-2, np=1; j!=(uint)-1; j--) {
                if (resp[j]==')') 
                    np++;
                else if (resp[j]=='(') {
                    np--;
                    if (np==0) break;
                }
            }
            if (np!=0){
                cout << "input must contain balanced parentheses\n";
                continue;
            }
            for (i=j; i>0 && strchr(" \t",resp[i-1]); i--) ;
            if (i==0){
                cout << "input must contain a non-empty coordinate name\n";
                continue;
            }
            name = resp.substr(0,i);
            unit = resp.substr(j+1, n-j-2);
            return;
        }
    }
}


string CCoord::str( int maxlen ) const
{
    string ret = name + " (" + unit + ")";
    if ( maxlen==0 || ret.length()<=maxlen )
        return ret;
    ret = name;
    if ( ret.length()<=maxlen )
        return ret;
    if( maxlen>3 )
        return name.substr(0,maxlen-2) + "..";
    return name.substr(0,maxlen);
}

string CCoord::ps_str() const // two blanks instead of one, and no unit ""
{
    return name + ( unit!="" ? ("  (" + unit + ")") : "");
}

//! Format for saving in a01 format.

string CCoord::to_a01() const
{
    return name + "\t" + unit;
}

string CCoord::load_a01(string in)
{
    string::size_type t1, t2;
    if(!in.size()) {
        name = "";
        unit = "";
        return "";
    }
    if ((t1 = in.find('\t'))==string::npos) {
        name = in;
        unit = "";
        return "";
    }
    name = in.substr(0,t1);
    if ((t2 = in.find('\t', t1+1))==string::npos) {
        unit = in.substr(t1+1);
        return "";
    }
    unit = in.substr(t1+1,t2-t1);
    return in.substr(t2+1);
}

//**************************************************************************//
//  outside CCoord                                                          //
//**************************************************************************//

ostream& operator<< (ostream &s, CCoord C)
{
    return s << C.str();
}

string strg(const CCoord& C) { return C.str(); };

//**************************************************************************//
//  class CParam                                                            //
//**************************************************************************//

// string CParam::str() const
// {
//     return strg(val) + string(" ") + Co.str();
// }
