//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* mem.cpp: online memory                                                 *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <algorithm>

#include <ask_simple_value.h>

using namespace std;

#include "mystd.h"
#include "olf.h"
#include "mem.h"


namespace NOlm { // on-line memory (in olm.h predefined as extern)
    // the data:
    vector<POlo> MOM;

    // the current selection:
    class CList FSel;
    class CList FSelOld;
    class CList FSelNew;
    bool overwrite;
};


//! Receive new file selection.

void NOlm::SelPush( CList& _FSel )
{
    FSelOld = FSel;
    FSel = _FSel;
}


//! Go back to previous file selection.

void NOlm::SelPop()
{
    FSel = FSelOld;
    FSelOld = CList();
}


//! Assert that file selection is not empty.

void NOlm::SelAssert()
{
    if( !FSel.size() )
        throw string( "empty operand file list" );
}


//! Append copy of file *olo to online memory.

void NOlm::OloAdd( POlo f, int k )
{
    if( overwrite && k!=-1 ){
        // replace input file
        if( k>=MOM.size() )
            throw string( "BUG: inconsistent k in OlfAdd" );
        MOM[k] = f;
    } else {
        // append at end of online memory
        MOM.push_back( f );
        FSelNew.append( MOM.size()-1 );
    }
}

void NOlm::OloAdd( POld fd, int k ) { OloAdd( (POlo)fd, k ); }
void NOlm::OloAdd( POlc fc, int k ) { OloAdd( (POlo)fc, k ); }


//! Remove selected files from online memory.

void NOlm::OlfDel()
{
    for (uint i = FSel.size()-1; i!=(uint)-1; --i) {
        MOM.erase(MOM.begin()+FSel.V[i]);
    }
    SelPop();
    SelPop();
    if( MOM.size() > 0 )
        FSel = CList(0,MOM.size()-1);
    else
        FSel = CList();
}


//! Append copy of selected files to online memory.

void NOlm::OlfCopy()
{
    IterateO fiter;
    POlo fin;
    while ( (fin=fiter()) ){
        POlo fout( fin->new_olo() );
        fout->copy_mainvec( fin );
        OloAdd( fout );
    }
}


//! Move files to end of online memory.

void NOlm::OlfExchange()
{
    FSelOld = FSel;
    FSel = CList();
    vector<POlo> aux;
    for (uint i=FSelOld.size()-1; i!=(uint)-1; --i) {
        aux.push_back( MOM[FSelOld.V[i]] );
        MOM.erase( MOM.begin()+FSelOld.V[i] );
    }
    for ( uint i=0; i<FSelOld.size(); ++i ){
        MOM.push_back( aux[i] );
        FSel.append( MOM.size()-1 );
    }
}


//! Interactive spectrum selection.

void NOlm::JSelAsk( string quest, string *jSel, CList *jLis )
{
    if ( FSel.size()<=0 )
        throw string( "query spectrum selection: no files to operate on" );

    IterateO fiter;
    POlo f;
    uint nJmax = 0;
    while ( (f=fiter()) )
        nJmax = max( nJmax, f->nJ() );

    if ( nJmax==1 )
        *jLis = CList("0");
    else {
        *jSel = wask( quest, *jSel );
        *jLis = CList ( *jSel, 0, nJmax-1 );
    }
}

