#include <stdlib.h>
#include <math.h>
#include <fftw3.h>
#include <iostream>
#include <string>
#include <vector>

#include <ask_simple_value.h>

using namespace std;

#include "mystd.h"
#include "olf.h"
#include "mem.h"
#include "special.h"

namespace NSpecial {
    void q_eval( double *par, int m_dat, double *fvec, void *data, int *info);
    void lm_print(int n_par, double *par, int m_dat, double *fvec,
                  void *data, int iflag, int iter, int nfev);
    double q_kernel( double t, double *p );
}

void NSpecial::FourierCosine() 
{
    fftw_plan plan;
    int n, nold=-1;
    
    NOlm::IterateD fiter;
    POld fin;
    while( (fin=fiter()) ) {
        POld fout( fin->new_old() );
        fout->lDoc.push_back("Fourier Cosine");

        cerr << "WARNING: coordinates not changed\n";
        cerr << "WARNING: x not changed\n";
        cerr << "WARNING: dy not calculated\n";
        // *** set coordinates ***
//                olf.xco = CCoord("z", "m");
//                olf.yco = CCoord("B_dim", "B-unit");

        for ( uint j=0; j<fin->nJ(); j++ ) {
            PSpec sin = fin->VS(j);
            PSpec sout( new CSpec( *sin ) );
            
            n = sin->size();
                        
            plan = fftw_plan_r2r_1d(
                n, &(sin->y[0]), &(sout->y[0]),
                FFTW_REDFT00,  FFTW_ESTIMATE);
            nold = n; // currently not used
            fftw_execute( plan );

            sout->dy.clear();
            for ( uint i=0; i<n; ++i ) {
                sout->y[i] /= sqrt(2.*n);
            }

            fout->V.push_back(sout);
        }
        NOlm::OloAdd( fout );
    }
}

void NSpecial::Test()
{
    // diagonalize finite chain force matrix

    POld fout( new COld );
    
    int N = iask("Order N");
    if ( N<2 )
         return;
         
    fout->name = "EV_" +strg(N);
    fout->xco = CCoord("n", "");
    fout->yco = CCoord("v_n", "");
    fout->ZCo.push_back( CCoord("p", "") );
    fout->ZCo.push_back( CCoord("nor", "") );

    for ( int p=0; p<N; ++p ){ 
        PSpec S( new CSpec );

        S->z.push_back( p );

        S->resize(N,false);
        for( uint n=0; n<N; ++n )
            S->x[n] = n;

        /// eigenvalue:
        double lambda = -2 + 2*cos(p*M_PI/N);

        // recursive calculation of eigenvector:
        S->y[0] = 1;
        S->y[1] = 1+lambda;
        for( uint n=2; n<N; ++n )
            S->y[n] = -S->y[n-2] + (2+lambda)*S->y[n-1];

        // normalize:
        double nor = 0;
        for( uint n=0; n<N; ++n )
            nor += S->y[n] * S->y[n];
        printf( "p=%i => check sum = %g, norm = %g\n",
                p, S->y[N-2]-(1+lambda)*S->y[N-1], nor );
        S->z.push_back( nor );
        for( uint n=0; n<N; ++n )
            S->y[n] /= sqrt(nor);

        fout->V.push_back(S);
    }
    NOlm::OloAdd( fout );
}
